package jWebCam;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.Properties;

import javafx.scene.canvas.Canvas;
import javafx.scene.image.PixelFormat;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritablePixelFormat;
import javafx.stage.DirectoryChooser;
import javafx.stage.FileChooser;
import uk.co.caprica.vlcj.binding.LibVlc;
import uk.co.caprica.vlcj.component.DirectMediaPlayerComponent;
import uk.co.caprica.vlcj.player.MediaPlayer;
import uk.co.caprica.vlcj.player.direct.BufferFormat;
import uk.co.caprica.vlcj.player.direct.BufferFormatCallback;
import uk.co.caprica.vlcj.player.direct.DirectMediaPlayer;
import uk.co.caprica.vlcj.player.direct.format.RV32BufferFormat;
import uk.co.caprica.vlcj.runtime.RuntimeUtil;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

import com.sun.jna.Memory;
import com.sun.jna.Native;
import com.sun.jna.NativeLibrary;
@Version(0.01f)
@Author("Daestrum")
@ShortName("jWebCam")
public class JWebCam<JImage> {
    private PixelWriter pixelWriter;
    private DirectMediaPlayerComponent mediaPlayerComponent;
    private static int WIDTH = 640;
    private static int HEIGHT = 480;
    private WritablePixelFormat<ByteBuffer> pixelFormat;
    private Canvas canvas;
    private MediaPlayer mp;
    private String vlcDir=null;
    private String webCam;

    public JWebCam() {
	}
	
    public void Initialize(Canvas canvas,String device,String media){
		retrieveProps();
		if (vlcDir == null){
		DirectoryChooser directoryChooser = new DirectoryChooser();
		directoryChooser.setTitle("Please find VideoLan VLC directory");
    	vlcDir = directoryChooser.showDialog(canvas.getParent().getScene().getWindow()).getAbsolutePath();
		}
		if (vlcDir!=""){
			saveProps();
		}
		if (webCam==""){
			webCam = media;
		}
    	WIDTH = (int) canvas.getWidth();
		HEIGHT = (int) canvas.getHeight();
		NativeLibrary.addSearchPath(
                RuntimeUtil.getLibVlcLibraryName(), vlcDir
                //RuntimeUtil.getLibVlcLibraryName(), "C:/Program Files/VideoLAN/VLC"
            );
        Native.loadLibrary(RuntimeUtil.getLibVlcLibraryName(), LibVlc.class);
		this.canvas = canvas;
        pixelWriter = this.canvas.getGraphicsContext2D().getPixelWriter();
        pixelFormat = PixelFormat.getByteBgraInstance();
        mediaPlayerComponent = new TestMediaPlayerComponent();
        mp = mediaPlayerComponent.getMediaPlayer();
        //mp.playMedia("dshow://","vdev=\"Live! Cam Vista IM\"");
        mp.playMedia(device,media);
	}
	public void release(){
		mediaPlayerComponent.release();
	}
	public void pause(){
		mp.pause();
	}
	public void resume(){
		mp.play();
	}
	public void stop(){
		mp.stop();
	}
	public void grab(String filename){
		File f = new File(filename);
		mp.saveSnapshot(f);
	}
    private class TestMediaPlayerComponent extends DirectMediaPlayerComponent {
        @Override
        public void display(DirectMediaPlayer mediaPlayer, Memory[] nativeBuffers, BufferFormat bufferFormat) {
            Memory nativeBuffer = nativeBuffers[0];
            ByteBuffer byteBuffer = nativeBuffer.getByteBuffer(0, nativeBuffer.size());
            pixelWriter.setPixels(0, 0, bufferFormat.getWidth(), bufferFormat.getHeight(), pixelFormat, byteBuffer, bufferFormat.getPitches()[0]);
        }

        public TestMediaPlayerComponent() {
            super(new TestBufferFormatCallback());
        }
    }
    private class TestBufferFormatCallback implements BufferFormatCallback {
        @Override
        public BufferFormat getBufferFormat(int sourceWidth, int sourceHeight) {
            canvas.setWidth(WIDTH);
            canvas.setHeight(HEIGHT);
            return new RV32BufferFormat(WIDTH, HEIGHT);
        }
    }
    private void saveProps(){
		OutputStream output = null;
		Properties prop = new Properties();
		try {
			output = new FileOutputStream("jWebCam.properties");
			if (vlcDir!=null){
			prop.setProperty("VLCLocation", vlcDir);
			prop.store(output, null);
			}
		} catch (IOException io) {
			io.printStackTrace();
		} finally {
			if (output != null) {
				try {
					output.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
	 
		}
    }
    private void retrieveProps(){
    	Properties prop = new Properties();
    	InputStream input = null;
    	  File f = new File("jWebCam.properties");
          if(!f.exists()) return;
    	try {
    		input = new FileInputStream("jWebCam.properties");
    		prop.load(input);
    		vlcDir = prop.getProperty("VLCLocation");
    	} catch (IOException ex) {
    		System.out.println(ex.getMessage());
    	} finally {
    		if (input != null) {
    			try {
    				input.close();
    			} catch (IOException e) {
    				e.printStackTrace();
    			}
    		}
    	}
          
		
    }
    
}
